// This is the basic program that registers itself with jack and sets
// up the ports for the data that will be processed remotely.

///////////////////////////////////////////////////////////////////////////////
//                  includes, defines, declaratios etc.
///////////////////////////////////////////////////////////////////////////////
#include <string> // used by myipaddr.hh
#include <sstream> // used in this file
#include <iostream> // used in this file
#include <getopt.h>

// These are needed by mysocket.hh:
#include <sys/time.h>
#include <time.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#include <jack/jack.h>

#include "myipaddr.hh"
#include "mysocket.hh"
#include "chain.hh"

// Since we don't process the program arguments inside main(), we need
// these to exchange data between parseargs() and main(); ugh! Passing
// pointers or references around doesn't seem better tho...
myIPAddress peerIP, *bindToPorts, *sendToPorts;
int inputs, outputs;


///////////////////////////////////////////////////////////////////////////////
//                             Functions
///////////////////////////////////////////////////////////////////////////////

void
usage (char* progname) {
	cerr << "syntax: " << progname;
	cerr << " -in number_of_jack_inputs -out number_of_jack_outputs ";
	cerr << "-remoteip ipnumber local_ports remote_ports" << endl;
}

void
parseargs (int argc, char** argv)
{
	// These are the options we recognize
	static struct option longOptions[] = {
		{ "in", 1, 0, 'i' },
		{ "out", 1, 0, 'o' },
		{ "remoteip", 1, 0, 'r' },
		{ 0, 0, 0, 0 } // "Terminator"
	};

	inputs = outputs = 0;
	while (1) {
		int option_index = 0;
		int c = getopt_long_only (argc, argv, \
			"i:o:r:", longOptions, &option_index);
		if (c == -1) // No more options
			break;
		switch (c) {
			case 'i':
				inputs = atoi (optarg);
				sendToPorts = new myIPAddress[inputs];
				break;
			case 'o':
				outputs = atoi (optarg);
				bindToPorts = new myIPAddress[outputs];
				break;
			case 'r':
				peerIP.IPAddress (optarg);
				break;
			default:
				cerr << "unknown option" << endl;
		}
	}

	// All arguments are mandatory
	if (argc != 7 + inputs + outputs) {
		cerr << "incorrect number of option options" << endl;
		usage (argv[0]);
		exit (1);
	}

	// Remaining args are the local and remote ports;
	// put them in their respective arrays.
	while (optind < argc) {
		for (int i = 0; i < outputs; i++) {
			bindToPorts[i].port (atoi(argv[optind++]));
		}
		for (int i = 0; i < inputs; i++) {
			sendToPorts[i].port (atoi(argv[optind++]));
			sendToPorts[i].IPAddress (peerIP.IPAddress());
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
//                               Main
///////////////////////////////////////////////////////////////////////////////

int main (int argc, char** argv) {

	// Remove the leading path from our name
	if (strrchr(argv[0], '/')) {
		argv[0] = strrchr(argv[0], '/');
		argv[0]++;
	}

	parseargs (argc, argv);

	stringstream chainName;
	chainName << argv[0] << "-" << getpid() << flush;
	chain myChain (chainName.str().c_str(), peerIP);
	myChain.createPorts (inputs, outputs);
	myChain.createOutputSockets (sendToPorts);
	myChain.createInputSockets (bindToPorts);
	myChain.activate ();
	sleep (10000);
	return 0;
}
