///////////////////////////////////////////////////////////////////////////////
//             includes, defines, declarations etc.
///////////////////////////////////////////////////////////////////////////////
#include <string> // used by myipaddr.hh
#include <sstream> // used in this file
#include <iostream> // used in this file

// These are needed by mysocket.hh:
#include <sys/time.h>
#include <time.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>


#include "myipaddr.hh"
#include "mysocket.hh"
#include <jack/jack.h>
#include "chain.hh"
#include "slave_driver.hh"



// This is only used when we're running as the driver; differently from the
// "normal" callbacks, the callbacks provided be jack to the driver don't
// provide for passing arbitrary data to the callback function. So, since
// the callbacks are C functions, we have no way of receiving a pointer to
// the slaveChain object we are using. But since there's only one of those
// when we're running as a driver, this pointer solves the problem.
// FIXME - probably not needed - we put the pointer in the driver struct
// static chain* theSlaveChain;


///////////////////////////////////////////////////////////////////////////////
// These are the callbacks that will be registered with JACK when we are
// running as the driver; they call the appropriate methods on the
// slaveChain object.
///////////////////////////////////////////////////////////////////////////////

typedef struct {
	JACK_DRIVER_DECL

	slaveChain *chain;

} network_driver_t;

jack_driver_t*
driver_initialize (int argc, char** argv)
{

	myIPAddress *remoteIPs;
	myIPAddress peerIP, localIP;
	int inputs = outputs = j = 0;
	if (argc >= 9) {
		for (int i = 1; i < argc; i++) {
			if (argv[i][0] == '-') {
				if (strcasecmp (argv[i][1], "ip") == 0) {
					peerIP.IPAddress (argv[i+1]);
					i++;
				}
				if (strcasecmp (argv[i][1], "port") == 0) {
					localIP.port (atoi(argv[i+1]));
					i++;
				}
				if (strcasecmp (argv[i][1], "inputs") == 0) {
					inputs = atoi(argv[i+1]);
					i++;
				}
				if (strcasecmp (argv[i][1], "outputs") == 0) {
					outputs = atoi(argv[i+1]);
					i++;
					remoteIPs = new myIPAddress[outputs];
				}
			} else {
				remoteIPs[j].port (atoi(argv[i]));
				remoteIPs[j].IPAddress (peerIP.IPAddress());
				j++;
			}
		}
	} else {
		cerr << "syntax: -ip peerIP -port port_to_bind ";
		cerr << "-inputs number_of_input_sockets ";
		cerr << "-outputs number_of_output_sockets";
		cerr << "peerport peerport...";
		cerr << endl;
	}

	period_usecs = MAX_CICLE_TIME;
	driver = (network_driver_t *) calloc (1, sizeof (network_driver_t));
	jack_driver_init ((jack_driver_t *) driver);

	driver->attach = (JackDriverAttachFunction) driver_attach;
	driver->detach = (JackDriverDetachFunction) driver_detach;
	driver->wait = (JackDriverWaitFunction) driver_wait;
	driver->process = (JackDriverProcessFunction) driver_process;
	driver->start = (JackDriverStartFunction) driver_audio_start;
	driver->stop = (JackDriverStopFunction) driver_audio_stop;

	driver->chain = new slaveChain ("driver", IP);

	// the number of input ports is the number of output sockets
	// and vice-versa.
	driver->chain->createPorts (outputs, inputs);
	driver->chain->createOutputSockets (remoteIPs);
	driver->chain->createInputSockets();
	myIPAddress* localIPs = new myIPAddress[inputs];
	driver->chain->listInputSockets (localIPs);

	cout << "The listening ports are: ";
	for (int i = 0; i < inputs - 1; i++) {
		cout << " " << localIPs[i].port() << ",";
	}
	cout << " " << localIPs[inputs -1].port() << "." << endl;
	cout << "Press enter to continue" << endl;
	cin.get();
}
